/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.story

import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.network.ApiCallback
import com.kakao.sdk.common.network.ApiFactory
import com.kakao.sdk.story.model.*
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.io.File

/**
 * @author kevin.kang. Created on 2019-11-05..
 */
class StoryApiClient(
    private val storyApi: StoryApi = ApiFactory.kapiWithOAuth.create(StoryApi::class.java)
) {

    fun isStoryUser(callback: (isStoryUser: Boolean?, error: Throwable?) -> Unit) {
        storyApi.isStoryUser()
            .enqueue(object: ApiCallback<StoryUserResult>() {
                override fun onComplete(model: StoryUserResult?, error: Throwable?) {
                    callback(model?.isStoryUser, error)
                }
            })
    }

    fun profile(secureResource: Boolean? = true, callback: (profile: StoryProfile?, error: Throwable?) -> Unit) {
        storyApi.profile(secureResource)
            .enqueue(object: ApiCallback<StoryProfile>() {
                override fun onComplete(model: StoryProfile?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun story(id: String, callback: (story: Story?, error: Throwable?) -> Unit) {
        storyApi.story(id)
            .enqueue(object: ApiCallback<Story>() {
                override fun onComplete(model: Story?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun stories(callback: (stories: List<Story>?, error: Throwable?) -> Unit) {
        storyApi.stories()
            .enqueue(object: ApiCallback<List<Story>>() {
                override fun onComplete(model: List<Story>?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun postNote(
        content: String,
        permission: Story.Permission = Story.Permission.PUBLIC,
        enableShare: Boolean = true,
        androidExecParams: Map<String, String>? = null,
        iosExecParams: Map<String, String>? = null,
        androidMarketParams: Map<String, String>? = null,
        iosMarketParams: Map<String, String>? = null,
        callback: (storyPostResult: StoryPostResult?, error: Throwable?) -> Unit
    ) {
        storyApi.postNote(content, permission, enableShare, androidExecParams, iosExecParams, androidMarketParams, iosMarketParams)
            .enqueue(object: ApiCallback<StoryPostResult>() {
                override fun onComplete(model: StoryPostResult?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun postPhoto(
        images: List<String>,
        content: String,
        permission: Story.Permission = Story.Permission.PUBLIC,
        enableShare: Boolean = true,
        androidExecParams: Map<String, String>? = null,
        iosExecParams: Map<String, String>? = null,
        androidMarketParams: Map<String, String>? = null,
        iosMarketParams: Map<String, String>? = null,
        callback: (storyPostResult: StoryPostResult?, error: Throwable?) -> Unit
    ) {
        storyApi.postPhoto(KakaoJson.toJson(images), content, permission, enableShare, androidExecParams, iosExecParams, androidMarketParams, iosMarketParams)
            .enqueue(object: ApiCallback<StoryPostResult>() {
                override fun onComplete(model: StoryPostResult?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun postLink(
        linkInfo: LinkInfo,
        content: String,
        permission: Story.Permission = Story.Permission.PUBLIC,
        enableShare: Boolean = true,
        androidExecParams: Map<String, String>? = null,
        iosExecParams: Map<String, String>? = null,
        androidMarketParams: Map<String, String>? = null,
        iosMarketParams: Map<String, String>? = null,
        callback: (storyPostResult: StoryPostResult?, error: Throwable?) -> Unit
    ) {
        storyApi.postLink(linkInfo, content, permission, enableShare, androidExecParams, iosExecParams, androidMarketParams, iosMarketParams)
            .enqueue(object: ApiCallback<StoryPostResult>() {
                override fun onComplete(model: StoryPostResult?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun delete(id: String, callback: (error: Throwable?) -> Unit) {
        storyApi.delete(id)
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    callback(error)
                }
            })
    }

    fun linkInfo(url: String, callback: (linkInfo: LinkInfo?, error: Throwable?) -> Unit) {
        storyApi.linkInfo(url)
            .enqueue(object: ApiCallback<LinkInfo>() {
                override fun onComplete(model: LinkInfo?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun upload(images: List<File>, callback: (uploadedPaths: List<String>?, error: Throwable?) -> Unit) {
        storyApi.upload(
            images.mapIndexed { index, file ->
                MultipartBody.Part.createFormData(
                    "${Constants.FILE}_$index",
                    file.name,
                    RequestBody.create(MediaType.parse("image/*"), file)
                )
            })
            .enqueue(object: ApiCallback<List<String>>() {
                override fun onComplete(model: List<String>?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    companion object {
        val instance by lazy { StoryApiClient() }
    }
}