/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.talk

import android.net.Uri
import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.ApplicationInfo
import com.kakao.sdk.common.model.ContextInfo
import com.kakao.sdk.common.network.ApiCallback
import com.kakao.sdk.common.network.ApiFactory
import com.kakao.sdk.talk.model.*
import com.kakao.sdk.template.DefaultTemplate

class TalkApiClient(
    private val talkApi: TalkApi = ApiFactory.kapiWithOAuth.create(TalkApi::class.java),
    private val applicationInfo: ApplicationInfo = KakaoSdk.applicationContextInfo,
    private val contextInfo: ContextInfo = KakaoSdk.applicationContextInfo
) {

    fun profile(
        secureResource: Boolean? = true,
        callback: (profile: TalkProfile?, error: Throwable?) -> Unit
    ) {
        talkApi.profile(secureResource)
            .enqueue(object: ApiCallback<TalkProfile>() {
                override fun onComplete(model: TalkProfile?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun friends(
        secureResource: Boolean? = true,
        offset: Int? = null,
        limit: Int? = null,
        order: Order? = null,
        callback: (friends: Friends<Friend>?, error: Throwable?) -> Unit
    ) {
        talkApi.friends(secureResource, offset, limit, order)
            .enqueue(object: ApiCallback<Friends<Friend>>() {
                override fun onComplete(model: Friends<Friend>?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun sendCustomMemo(
        templateId: Long,
        templateArgs: Map<String, String>? = null,
        callback: (error: Throwable?) -> Unit
    ) {
        talkApi.sendCustomMemo(templateId, templateArgs)
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    callback(error)
                }
            })
    }

    fun sendDefaultMemo(
        template: DefaultTemplate,
        callback: (error: Throwable?) -> Unit
    ) {
        talkApi.sendDefaultMemo(template)
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    callback(error)
                }
            })
    }

    fun sendScrapMemo(
        requestUrl: String,
        templateId: Long? = null,
        templateArgs: Map<String, String>? = null,
        callback: (error: Throwable?) -> Unit
    ) {
        talkApi.sendScrapMemo(requestUrl, templateId, templateArgs)
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    callback(error)
                }
            })
    }

    fun sendCustomMessage(
        receiverUuids: List<String>,
        templateId: Long,
        templateArgs: Map<String, String>? = null,
        callback: (result: MessageSendResult?, error: Throwable?) -> Unit
    ) {
        talkApi.sendCustomMessage(KakaoJson.toJson(receiverUuids), templateId, templateArgs)
            .enqueue(object: ApiCallback<MessageSendResult>() {
                override fun onComplete(model: MessageSendResult?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun sendDefaultMessage(
        receiverUuids: List<String>,
        template: DefaultTemplate,
        callback: (result: MessageSendResult?, error: Throwable?) -> Unit
    ) {
        talkApi.sendDefaultMessage(KakaoJson.toJson(receiverUuids), template)
            .enqueue(object: ApiCallback<MessageSendResult>() {
                override fun onComplete(model: MessageSendResult?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun sendScrapMessage(
        receiverUuids: List<String>,
        requestUrl: String,
        templateId: Long? = null,
        templateArgs: Map<String, String>? = null,
        callback: (result: MessageSendResult?, error: Throwable?) -> Unit
    ) {
        talkApi.sendScrapMessage(KakaoJson.toJson(receiverUuids), requestUrl, templateId, templateArgs)
            .enqueue(object: ApiCallback<MessageSendResult>() {
                override fun onComplete(model: MessageSendResult?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun channels(
        publicIds: String? = null,
        callback: (relations: ChannelRelations?, error: Throwable?) -> Unit
    ) {
        talkApi.channels(publicIds)
            .enqueue(object: ApiCallback<ChannelRelations>() {
                override fun onComplete(model: ChannelRelations?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    /**
     * 카카오톡 채널을 추가하기 위한 URL 을 반환합니다. URL 을 브라우저나 웹뷰에서 로드하면 브릿지 웹페이지를 통해 카카오톡을 실행합니다.
     *
     * channelPublicId: 카카오톡 채널 홈 URL 에 들어간 {_영문}으로 구성된 고유 아이디입니다.
     * 홈 URL 은 카카오톡 채널 관리자센터 > 관리 > 상세설정 페이지에서 확인할 수 있습니다.
     *
     * @see com.kakao.sdk.common.util.KakaoCustomTabsClient
     */
    fun addChannelUrl(channelPublicId: String): Uri {
        return baseUri(appKey = applicationInfo.appKey, kaHeader = contextInfo.kaHeader)
            .path("$channelPublicId/${Constants.FRIEND}").build()
    }

    /**
     * 카카오톡 채널 1:1 대화방 실행을 위한 URL 을 반환합니다. URL 을 브라우저나 웹뷰에서 로드하면 브릿지 웹페이지를 통해 카카오톡을 실행합니다.
     *
     * channelPublicId: 카카오톡 채널 홈 URL 에 들어간 {_영문}으로 구성된 고유 아이디입니다.
     * 홈 URL 은 카카오톡 채널 관리자센터 > 관리 > 상세설정 페이지에서 확인할 수 있습니다.
     *
     * @see com.kakao.sdk.common.util.KakaoCustomTabsClient
     */
    fun channelChatUrl(channelPublicId: String): Uri {
        return baseUri(appKey = applicationInfo.appKey, kaHeader = contextInfo.kaHeader)
            .path("$channelPublicId/${Constants.CHAT}").build()
    }

    private fun baseUri(appKey: String, kaHeader: String): Uri.Builder {
        return Uri.Builder().scheme(com.kakao.sdk.common.Constants.SCHEME)
            .authority(KakaoSdk.serverHosts.channel)
            .appendQueryParameter(com.kakao.sdk.common.Constants.APP_KEY, appKey)
            .appendQueryParameter(Constants.KAKAO_AGENT, kaHeader)
            .appendQueryParameter(Constants.API_VER, Constants.API_VER_10)
    }

    companion object {
        val instance by lazy { TalkApiClient() }
    }
}